/*
    Copyright (C) 2001  bjk <bjk@arbornet.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <pwd.h>
#include <utmp.h>
#include "config.h"

#ifdef HAVE_GETSPNAM
#include <shadow.h>
#endif

#ifndef ALLPERMS
#define	ALLPERMS (S_ISUID|S_ISGID|S_ISTXT|S_IRWXU|S_IRWXG|S_IRWXO)
#endif

#ifndef NAME_MAX
#define NAME_MAX 255
#endif

#ifndef LINE_MAX
#define LINE_MAX 2048
#endif

#define VERSION "Userinfo 1.0"
#define COPYRIGHT "Copyright 2001 bjk <bjk@arbornet.org>"

#define MAXOPTIONS 28
#define MAXGECOSFIELDS 4
#define PW_FIELDS 10

#define TIMEBUF 64

/* structure definitions */
typedef struct gecos {
    char *name;
    char *office;
    char *officephone;
    char *homephone;
    char gecos_opts[MAXGECOSFIELDS];
} GECOS;

typedef struct mails {
    char lastread[TIMEBUF];
    char newstamp[TIMEBUF];
    char foldersize[10];
    char forward[LINE_MAX];
    char aliases[LINE_MAX];
} MAIL;

typedef struct logins {
    char stamp[TIMEBUF];
    char last[UT_LINESIZE + UT_HOSTSIZE + TIMEBUF];
    char duration[10];
    char host[UT_HOSTSIZE];
    char tty[UT_LINESIZE];
    unsigned int mesgstat;
    unsigned int online;
} LOGIN;

typedef struct groups {
    gid_t id;
    char name[32];
    struct groups *next;
} GROUP;

typedef struct infos {
    struct passwd *passwd;
#ifdef HAVE_GETSPNAM
    struct spwd *spwd;
#endif
    GROUP *group;
    GECOS *gecos;
    MAIL *mail;
    LOGIN *login;
    mode_t homedirmode;
} INFO;

/* external variables */
unsigned int verbose, is_stdin, usefile, delimchar;
unsigned int mail_info, login_info, passwd_info, group_info;
int passwd_opts[PW_FIELDS], mail_opts[MAXOPTIONS], login_opts[MAXOPTIONS];
int optspec[MAXOPTIONS];
char *gecosopts, tf[TIMEBUF];
int utmpfd;
FILE *aliasfp, *lastfp;
#ifdef HAVE_GETSPNAM
int amroot;
#endif

/* function prototypes */
char *itoa(int n);
void mailinfo(INFO *, const char *arg);
void outputinfo(INFO *, const char *arg);
void passwdinfo(INFO *);
void logininfo(INFO *, const char *arg);
char *stripstr(char *str, char *rm);
char *stamp(time_t epoch, const char *format);
